<?php

/**
 * @file
 * Contains \Drupal\search_api_hashtag\Tests\HashtagProcessorTest.
 */

namespace Drupal\search_api_hashtag\Tests;

use Drupal\node\Entity\Node;
use Drupal\node\Entity\NodeType;
use Drupal\search_api\Tests\Processor\ProcessorTestBase;
use Drupal\search_api\Utility;

/**
 * Tests the "Hashtag" processor.
 *
 * @group search_api
 *
 * @see \Drupal\search_api_hashtag\Plugin\search_api\processor\HashtagProcessor
 */
class HashtagProcessorTest extends ProcessorTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = array('node', 'search_api','search_api_db', 'search_api_test_backend', 'search_api_hashtag');

  /**
   * Stores the processor to be tested.
   *
   * @var \Drupal\search_api_hashtag\Plugin\search_api\processor\HashtagProcessor
   */
  protected $processor;

  /**
   * The nodes created for testing.
   *
   * @var \Drupal\node\Entity\Node[]
   */
  protected $nodes;

  /**
   * Performs setup tasks before each individual test method is run.
   */
  public function setUp() {
    parent::setUp('hashtag');

    // Create a node type for testing.
    $type = NodeType::create(array('type' => 'page', 'name' => 'page'));
    $type->save();

    $field_storage = entity_create('field_storage_config', array(
      'field_name' => 'body',
      'entity_type' => 'node',
      'type' => 'text',
      'settings' => array(
        'max_length' => '60',
      )
    ));
    $field_storage->save();
    entity_create('field_config', array(
      'field_storage' => $field_storage,
      'bundle' => 'page',
    ))->save();

    // Create a node with attached comment.
    $this->nodes[0] = Node::create(array('status' => NODE_PUBLISHED, 'type' => 'page', 'title' => '#test title'));
    $this->nodes[0]->save();

    $this->nodes[1] = Node::create(array('status' => NODE_PUBLISHED, 'type' => 'page', 'title' => 'some #title', 'body' => '#mybody'));
    $this->nodes[1]->save();

    $this->nodes[2] = Node::create(array('status' => NODE_NOT_PUBLISHED, 'type' => 'page', 'title' => '#Other #Title', 'body' => '#other body'));
    $this->nodes[2]->save();

    // Enable the processor's field on the index.
    $fields = $this->index->getOption('fields');
    $fields['entity:node/body'] = array(
      'type' => 'text',
    );
    $fields['entity:node/title'] = array(
      'type' => 'string',
    );
    $this->index->setOption('fields', $fields);

    // Also index users, to verify that they are unaffected by the processor.
    $this->index->set('datasources', array('entity:node'));
    $this->index->save();

    $this->index = entity_load('search_api_index', $this->index->id(), TRUE);
  }

  /**
   * Tests hashtag extraction.
   */
  public function testHashtagExtractor() {

    $base_items = array();
    foreach ($this->nodes as $node) {
      $base_items[] = array(
        'datasource' => 'entity:node',
        'item' => $node->getTypedData(),
        'item_id' => $node->id(),
        'text' => $this->randomMachineName(),
        'search_api_language' => 'en',
        'title' => $node->title->value,
        'body' => $node->body->value
      );
    }

    // Set proper configuration for the tested processor.
    $config = $this->processor->getConfiguration();
    $config['fields'] = array(
      'entity:node/body',
      'entity:node/title',
    );
    $config['lowercase'] = 0;
    $this->processor->setConfiguration($config);

    $items = $this->generateItems($base_items);
    $this->processor->preprocessIndexItems($items);
    $expected_values = array(
      'entity:node/1' => array(
        'test',
      ),
      'entity:node/2' => array(
        'mybody',
        'title',
      ),
      'entity:node/3' => array(
        'other',
        'Other',
        'Title',
      ),
    );

    $this->assertHashtag($items, $expected_values);

    // Set proper configuration for the tested processor.
    $config = $this->processor->getConfiguration();
    $config['fields'] = array(
      'entity:node/body',
    );
    $config['lowercase'] = 0;
    $this->processor->setConfiguration($config);

    $items = $this->generateItems($base_items);
    $this->processor->preprocessIndexItems($items);

    $expected_values = array(
      'entity:node/1' => array(
      ),
      'entity:node/2' => array(
        'mybody',
      ),
      'entity:node/3' => array(
        'other',
      ),
    );

    $this->assertHashtag($items, $expected_values);

    // Set proper configuration for the tested processor.
    $config = $this->processor->getConfiguration();
    $config['fields'] = array(
      'entity:node/body',
      'entity:node/title',
    );
    $config['lowercase'] = 1;
    $this->processor->setConfiguration($config);

    $items = $this->generateItems($base_items);
    $this->processor->preprocessIndexItems($items);

    $expected_values = array(
      'entity:node/1' => array(
        'test',
      ),
      'entity:node/2' => array(
        'mybody',
        'title',
      ),
      'entity:node/3' => array(
        'other',
        'title',
      ),
    );

    $this->assertHashtag($items, $expected_values);
  }

  /**
   * Asserts extracted hashtag values are the expected.
   *
   * @param $items
   * @param $expected_values
   */
  protected function assertHashtag($items, $expected_values) {
    foreach ($items as $key => $item) {
      list(, $nid) = Utility::splitCombinedId($key);
      $field = $item->getField('search_api_hashtag');
      $this->assertEqual($field->getType(), 'string', 'Node item ' . $nid . ' rendered value is identified as text.');
      $values = $field->getValues();
      $this->assertEqual($values, $expected_values[$key]);
    }
  }

}
